class JobInputList extends ResourceList {
  static autoInit() {
    for (let jobInputListElement of document.querySelectorAll('.job-input-list:not(.no-autoinit)')) {
      new JobInputList(jobInputListElement);
    }
  }

  constructor(listContainerElement, options = {}) {
    super(listContainerElement, options);
    this.listjs.list.addEventListener('click', (event) => {this.onClick(event)});
    this.isInitialized = false;
    this.userId = listContainerElement.dataset.userId;
    this.jobId = listContainerElement.dataset.jobId;
    app.subscribeUser(this.userId).then((response) => {
      app.socket.on('PATCH', (patch) => {
        if (this.isInitialized) {this.onPatch(patch);}
      });
    });
    app.getUser(this.userId).then((user) => {
      this.add(Object.values(user.jobs[this.jobId].inputs));
      this.isInitialized = true;
    });
  }

  get item() {
    return `
      <tr class="clickable hoverable">
        <td><span class="filename"></span></td>
        <td class="right-align">
          <a class="action-button btn-floating waves-effect waves-light" data-action="download"><i class="material-icons">file_download</i></a>
        </td>
      </tr>
    `.trim();
  }

  get valueNames() {
    return [
      {data: ['id']},
      {data: ['creation-date']},
      'filename'
    ];
  }

  initListContainerElement() {
    if (!this.listContainerElement.hasAttribute('id')) {
      this.listContainerElement.id = Utils.generateElementId('job-input-list-');
    }
    let listSearchElementId = Utils.generateElementId(`${this.listContainerElement.id}-search-`);
    this.listContainerElement.innerHTML = `
      <div class="input-field">
        <i class="material-icons prefix">search</i>
        <input id="${listSearchElementId}" class="search" type="text"></input>
        <label for="${listSearchElementId}">Search job input</label>
      </div>
      <table>
        <thead>
          <tr>
            <th>Filename</th>
            <th></th>
          </tr>
        </thead>
        <tbody class="list"></tbody>
      </table>
      <ul class="pagination"></ul>
    `.trim();
  }

  mapResourceToValue(jobInput) {
    return {
      'id': jobInput.id,
      'creation-date': jobInput.creation_date,
      'filename': jobInput.filename
    };
  }

  sort() {
    this.listjs.sort('filename', {order: 'asc'});
  }

  onClick(event) {
    let jobInputElement = event.target.closest('tr');
    if (jobInputElement === null) {return;}
    let jobInputId = jobInputElement.dataset.id;
    if (jobInputId === undefined) {return;}
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'download' : actionButtonElement.dataset.action;
    switch (action) {
      case 'download': {
        window.location.href = `/jobs/${this.jobId}/inputs/${jobInputId}/download`;
        break;
      }
      default: {
        break;
      }
    }
  }
}
