class UserList extends ResourceList {
  static autoInit() {
    for (let userListElement of document.querySelectorAll('.user-list:not(.no-autoinit)')) {
      new UserList(userListElement);
    }
  }

  constructor(listContainerElement, options = {}) {
    super(listContainerElement, options);
    this.listjs.list.addEventListener('click', (event) => {this.onClick(event)});
  }

  get item() {
    return `
      <tr class="clickable hoverable">
        <td><span class="id-1"></span></td>
        <td><span class="username"></span></td>
        <td><span class="email"></span></td>
        <td><span class="last-seen"></span></td>
        <td><span class="role"></span></td>
        <td class="right-align">
          <a class="action-button btn-floating red waves-effect waves-light" data-action="delete"><i class="material-icons">delete</i></a>
          <a class="action-button btn-floating waves-effect waves-light" data-action="edit"><i class="material-icons">edit</i></a>
          <a class="action-button btn-floating waves-effect waves-light" data-action="view"><i class="material-icons">send</i></a>
        </td>
      </tr>
    `.trim();
  }

  get valueNames() {
    return [
      {data: ['id']},
      {data: ['member-since']},
      'email',
      'id-1',
      'last-seen',
      'role',
      'username'
    ];
  }

  initListContainerElement() {
    if (!this.listContainerElement.hasAttribute('id')) {
      this.listContainerElement.id = Utils.generateElementId('user-list-');
    }
    let listSearchElementId = Utils.generateElementId(`${this.listContainerElement.id}-search-`);
    this.listContainerElement.innerHTML = `
      <div class="input-field">
        <i class="material-icons prefix">search</i>
        <input id="${listSearchElementId}" class="search" type="text"></input>
        <label for="${listSearchElementId}">Search user</label>
      </div>
      <table>
        <thead>
          <tr>
            <th>Id</th>
            <th>Username</th>
            <th>Email</th>
            <th>Last seen</th>
            <th>Role</th>
            <th></th>
          </tr>
        </thead>
        <tbody class="list"></tbody>
      </table>
      <ul class="pagination"></ul>
    `.trim();
  }

  mapResourceToValue(user) {
    return {
      'id': user.id,
      'id-1': user.id,
      'username': user.username,
      'email': user.email,
      'last-seen': new Date(user.last_seen).toLocaleString('en-US'),
      'member-since': user.member_since,
      'role': user.role.name
    };
  }

  sort() {
    this.listjs.sort('member-since', {order: 'desc'});
  }

  onClick(event) {
    let userElement = event.target.closest('tr');
    if (userElement === null) {return;}
    let userId = userElement.dataset.id;
    if (userId === undefined) {return;}
    let actionButtonElement = event.target.closest('.action-button');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete': {
        Utils.deleteUserRequest(userId);
        if (userId === currentUserId) {window.location.href = '/';}
        break;
      }
      case 'edit': {
        window.location.href = `/admin/users/${userId}/edit`;
        break;
      }
      case 'view': {
        window.location.href = `/admin/users/${userId}`;
        break;
      }
      default: {
        break;
      }
    }
  }
}
